import typer

from cirun.cloud import cloud_app
from cirun.repo import repo_app

app = typer.Typer(
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
)

app.add_typer(repo_app, name="repo")
app.add_typer(cloud_app, name="cloud")

if __name__ == "__main__":
    app()



from typer.core import TyperGroup

from click import Context


class OrderCommands(TyperGroup):
    def list_commands(self, ctx: Context):
        """Return list of commands in the order appear."""
        return list(self.commands)



import pytest

from cirun import Cirun


def test_raise_error_when_key_not_set():
    with pytest.raises(ValueError):
        Cirun()


def test_not_raise_error_when_token_set():
    Cirun(token="cirun-token-foo-bar")



import os

import requests
from rich.console import Console

API_ENDPOINT = "https://cirun.io/api/v1"


class CirunAPIException(Exception):
    pass


def _print_error(response):
    error_console = Console(stderr=True, style="bold red")
    try:
        rjson = response.json()
    except requests.exceptions.JSONDecodeError:
        rjson = {
            "responseContent": response.content.decode()
        }
    error_console.rule("[bold red]")
    error_console.print(f"Error")
    error_console.print_json(data=rjson)
    error_console.log(f"Response Status Code: {response.status_code}")
    error_console.rule("[bold red]")
    return rjson


class Cirun:
    def __init__(self, token=None):
        self.token = token
        self._get_credentials()

    def _get_credentials(self):
        if not self.token:
            try:
                token = os.environ['CIRUN_API_KEY']
                self.token = token
            except KeyError:
                raise ValueError("Could not find CIRUN_API_KEY in environment variables")

    def _headers(self):
        return {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.token}"
        }

    def _get(self, path, *args, **kwargs):
        return requests.get(f"{API_ENDPOINT}/{path}", headers=self._headers(), *args, **kwargs)

    def _post(self, path, *args, **kwargs):
        return requests.post(f"{API_ENDPOINT}/{path}", headers=self._headers(), *args, **kwargs)

    def get_repos(self, print_error=False):
        """Get all the repositories connected to cirun."""
        response = self._get("repo")
        if response.status_code not in [200, 201]:
            if print_error:
                return _print_error(response)
        return response.json()

    def set_repo(self, name, active=True, print_error=False):
        """Activate repository for Cirun"""
        data = {
            "repository": name,
            "active": active
        }
        response = self._post("repo", json=data)
        if response.status_code not in [200, 201]:
            if print_error:
                _print_error(response)
            return response.json()
        return response.json()

    def clouds(self, print_error=False):
        """Returns all the connected cloud"""
        response = self._get("cloud-connect")
        if response.status_code not in [200, 201]:
            if print_error:
                return _print_error(response)
        return response.json()

    def cloud_connect(self, name, credentials, print_error=False):
        """Connect a cloud provider to cirun"""
        data = {
            "cloud": name,
            "credentials": credentials
        }
        response = self._post("cloud-connect", json=data)
        if response.status_code not in [200, 201]:
            if print_error:
                _print_error(response)
            return response.json()
        return response.json()



import typer
from rich.console import Console

from cirun import Cirun
from cirun.utils import OrderCommands

repo_app = typer.Typer(
    cls=OrderCommands,
    help="Manage Repository Activation/Deactivation",
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
)
RepoName = typer.Argument(
    default=None,
    help=f"Repository Name, for example: cirunlabs/cirun",
    is_eager=True
)


@repo_app.command("list")
def list_():
    """Activate cirun on given repository"""
    cirun = Cirun()
    response_json = cirun.get_repos(print_error=True)
    console = Console(style="bold green")
    console.rule("[bold green]")
    console.print_json(data=response_json)
    console.rule("[bold green]")


@repo_app.command()
def add(name: str = RepoName):
    """Activate cirun on given repository"""
    cirun = Cirun()
    response_json = cirun.set_repo(name, active=True, print_error=True)
    console = Console(style="bold green")
    console.rule("[bold green]")
    console.print_json(data=response_json)
    console.rule("[bold green]")


@repo_app.command()
def remove(name: str = RepoName):
    """Deactivate cirun on given repository"""
    cirun = Cirun()
    response_json = cirun.set_repo(name, active=False, print_error=True)
    console = Console(style="bold green")
    console.rule("[bold green]")
    console.print_json(data=response_json)
    console.rule("[bold green]")



import json

import typer
from rich.console import Console

from cirun import Cirun
from cirun.utils import OrderCommands

cloud_app = typer.Typer(
    cls=OrderCommands,
    help="Manage cloud providers",
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
)

cloud_connect = typer.Typer(
    cls=OrderCommands,
    help="Connect cloud providers",
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich",
    context_settings={"help_option_names": ["-h", "--help"]},
)


cloud_app.add_typer(cloud_connect, name="connect")


def option(name, help, *args, **kwargs):
    return typer.Option(
        ...,
        name,
        help=help,
        show_default=False,
        *args, **kwargs
    )


@cloud_connect.command()
def aws(
        access_key=option("--access-key", help="AWS_ACCESS_KEY_ID"),
        secret_key=option("--secret-key", help="AWS_SECRET_ACCESS_KEY"),
):
    """Connect AWS to Cirun"""
    credentials = {
        "access_key": access_key,
        "secret_key": secret_key
    }
    _connect_cloud(name="aws", credentials=credentials)


@cloud_connect.command()
def azure(
        subscription_id=option("--subscription-id", help="Azure subscription_id"),
        tenant_id=option("--tenant-id", help="Azure tenant_id"),
        client_id=option("--client-id", help="Azure client_id"),
        client_secret=option("--client-secret", help="Azure client_secret"),
):
    """Connect Azure cloud to Cirun"""
    credentials = {
        "subscription_id": subscription_id,
        "tenant_id": tenant_id,
        "client_id": client_id,
        "client_secret": client_secret,
    }
    _connect_cloud(name="azure", credentials=credentials)


@cloud_connect.command()
def gcp(
        service_account_file=option("--key-file", help="GCP Service Account Key file",),
):
    """Connect GCP to Cirun"""
    service_account_txt = open(service_account_file, 'r').read()
    try:
        credentials = json.loads(service_account_txt)
    except json.JSONDecodeError:
        raise ValueError(f"Invalid key file: {service_account_file}")
    _connect_cloud(name="gcp", credentials=credentials)


@cloud_connect.command()
def openstack(
        username=option("--username", help="OpenStack username"),
        password=option("--password", help="OpenStack password"),
        auth_url=option("--auth-url", help="OpenStack auth_url"),
        project_id=option("--project-id", help="OpenStack project_id"),
        domain_id=option("--domain-id", help="OpenStack domain_id"),
        network=option("--network", help="OpenStack network"),
):
    """Connect Openstack to Cirun"""
    credentials = {
        "username": username,
        "password": password,
        "auth_url": auth_url,
        "project_id": project_id,
        "domain_id": domain_id,
        "network": network,
    }
    _connect_cloud(name="openstack", credentials=credentials)


def _connect_cloud(name, credentials):
    cirun = Cirun()
    response_json = cirun.cloud_connect(
        name=name,
        credentials=credentials,
        print_error=True
    )
    console = Console(style="bold green")
    console.rule("[bold green]")
    console.print_json(data=response_json)
    console.rule("[bold green]")
